/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.1 Ganglinienprognose
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.glprog.glprog;

import java.util.ArrayList;

import de.bsvrz.iav.gllib.gllib.dav.GanglinieMQ;
import de.bsvrz.sys.funclib.bitctrl.modell.kalender.objekte.EreignisTyp;

/**
 * Fasst Ganglinien zusammen, die zu einem gemeinsamen Ereignistyp gehören, welches entweder ein
 * Tagesereignis ist oder nicht. Handelt es sich um ein Tagesereignis, dann sind alle Ganglinien
 * absolut, sonst sind sie alle relativ (additiv oder multiplikativ).
 *
 * @author BitCtrl Systems GmbH, Schumann
 */
public class EreignisTypGanglinien extends ArrayList<GanglinieMQ> {

	/** Die ID für die Serialisierung. */
	private static final long serialVersionUID = 1L;

	/** Der Ereignistyp, auf das sich alle enthaltenen Ganglinien beziehen. */
	private final EreignisTyp ereignisTyp;

	/** Flag ob es sich hier um Tagesganglinien handelt. */
	private boolean tagesganglinie;

	/**
	 * Konstruiert die Ganglinienliste, in dem das Bezugsereignis festgelegt wird.
	 *
	 * @param ereignisTyp
	 *            der Ereignistyp, worauf sich alle Ganglinien der Liste beziehen.
	 */
	public EreignisTypGanglinien(final EreignisTyp ereignisTyp) {
		assert ereignisTyp != null : "Ereignistyp darf nicht null sein."; //$NON-NLS-1$
		this.ereignisTyp = ereignisTyp;
	}

	/**
	 * Gibt das Bezugsereignis der Ganglinienliste zurück.
	 *
	 * @return das Bezugsereignis.
	 */
	public EreignisTyp getEreignisTyp() {
		return ereignisTyp;
	}

	/**
	 * Gibt das Flag zurück, ob die Liste Ganglinien für ein Tagesereignis enthält.
	 *
	 * @return {@code true}, wenn die Ganglinien zu einem Tagesereignis gehören.
	 */
	public boolean isTagesganglinie() {
		return tagesganglinie;
	}

	/**
	 * Legt fest, ob es die Liste Ganglinien zu einem Tagesereignis sichert.
	 *
	 * @param tagesganglinie
	 *            {@code true}, wenn die Ganglinien zu einem Tagesereignis gehören.
	 */
	public void setTagesganglinie(final boolean tagesganglinie) {
		this.tagesganglinie = tagesganglinie;
	}

	@Override
	public EreignisTypGanglinien clone() {
		return (EreignisTypGanglinien) super.clone();
	}

	@Override
	public String toString() {
		String s;

		s = getClass().getSimpleName() + "["; //$NON-NLS-1$
		s += "ereignisTyp=" + ereignisTyp; //$NON-NLS-1$
		s += ", tagesganglinie=" + tagesganglinie; //$NON-NLS-1$
		s += ", Ganglinienanzahl=" + size(); //$NON-NLS-1$

		return s + "]"; //$NON-NLS-1$
	}

}
