/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.1 Ganglinienprognose
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.glprog.glprog;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import de.bsvrz.iav.gllib.gllib.modell.onlinedaten.OdPrognoseGanglinienAnfrage;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.ganglinien.objekte.ApplikationGanglinienPrognose;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Nimmt Anfragen vom Datenverteiler entgegen und startet deren Bearbeitung.
 *
 * @author BitCtrl Systems GmbH, Schumann
 */
public class Verwaltung implements DatensatzUpdateListener {

	/** Der Logger dieser Klasse. */
	private final Debug log = Debug.getLogger();

	private final ExecutorService threadPool;

	/**
	 * Initialisiert die eigentliche Anwendungslogik.
	 *
	 * @param glProg
	 *            die Ganglinienprognose.
	 * @param maxAnfragen
	 *            die maximale Anzahl von Anfragen, die gleichzeitig (als Thread) bearbeitet werden
	 *            sollen.
	 */
	public Verwaltung(final ApplikationGanglinienPrognose glProg, final int maxAnfragen) {
		threadPool = Executors.newFixedThreadPool(maxAnfragen);

		final OdPrognoseGanglinienAnfrage anfrage = glProg
				.getOnlineDatensatz(OdPrognoseGanglinienAnfrage.class);
		anfrage.setSenke(OdPrognoseGanglinienAnfrage.Aspekte.Anfrage.getAspekt(), true);
		anfrage.addUpdateListener(OdPrognoseGanglinienAnfrage.Aspekte.Anfrage.getAspekt(), this);
		LogTools.log(log, GlProgMsg.ConfigVerwaltungBereit, maxAnfragen);
	}

	/**
	 * Empfängt neue Anfragen und startet deren Bearbeitung.
	 */
	@Override
	public void datensatzAktualisiert(final DatensatzUpdateEvent event) {
		if (event.getDatum().isValid()) {
			final AnfrageInBearbeitung anfrage = new AnfrageInBearbeitung(
					(OdPrognoseGanglinienAnfrage.Daten) event.getDatum());
			threadPool.execute(anfrage);
		}
	}

}
