/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.1 Ganglinienprognose
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.glprog.glprog.util;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import de.bsvrz.iav.gllib.gllib.dav.GlProgAnfrage;
import de.bsvrz.iav.gllib.gllib.modell.parameter.PdGanglinienModellPrognose;
import de.bsvrz.iav.gllib.gllib.modell.parameter.PdGanglinienModellPrognose.Daten;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;

/**
 * Hilfsklasse für den Empfang des Prognoseparameters.
 *
 * <p>
 * Der Prognoseparameter kann direkt abgerufen werden mit {@link #getPrognoseParameter()}.
 * Zusätzlich kann man sich auf Änderung des Prognoseparameters als {@link PropertyChangeListener}
 * anmelden.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class PrognoseParameterHelper {

	/**
	 * Der Name der Property, unter der ein neuer Wert des Prognoseparameters veröffentlicht wird.
	 */
	public static final String PROP_PROGNOSE_PARAMETER = "prognoseParameter"; //$NON-NLS-1$

	private final DatensatzUpdateListener datensatzUpdateListener = new DatensatzUpdateListener() {

		@Override
		public void datensatzAktualisiert(final DatensatzUpdateEvent event) {
			if (event.getDatum().isValid()) {
				prognoseParameter = (Daten) event.getDatum();
				propertyChangeSupport.firePropertyChange(PROP_PROGNOSE_PARAMETER, null,
						prognoseParameter);
			}
		}

	};

	private final PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

	private PdGanglinienModellPrognose pdPrognoseParameter;
	private PdGanglinienModellPrognose.Daten prognoseParameter;
	private GlProgAnfrage anfrage;

	/**
	 * Gibt die Anfrage zurück, zu der die relevanten Ereignisse bestimmt werden sollen.
	 *
	 * @return die Anfrage.
	 */
	public GlProgAnfrage getAnfrage() {
		return anfrage;
	}

	/**
	 * Legt die Anfrage fest, zu der die relevanten Ereignisse bestimmt werden sollen.
	 *
	 * @param anfrage
	 *            die Anfrage.
	 */
	public void setAnfrage(final GlProgAnfrage anfrage) {
		this.anfrage = anfrage;

		pdPrognoseParameter = anfrage.getMessQuerschnitt()
				.getParameterDatensatz(PdGanglinienModellPrognose.class);
	}

	/**
	 * Meldet einen Property Change Listener an.
	 *
	 * @param l
	 *            der anzumeldende Listener.
	 */
	public void addPropertyChangeListener(final PropertyChangeListener l) {
		pdPrognoseParameter.addUpdateListener(datensatzUpdateListener);
		propertyChangeSupport.addPropertyChangeListener(l);
	}

	/**
	 * Meldet einen Property Change Listener wieder ab.
	 *
	 * @param l
	 *            der abzumeldende Listener.
	 */
	public void removePropertyChangeListener(final PropertyChangeListener l) {
		pdPrognoseParameter.removeUpdateListener(datensatzUpdateListener);
		propertyChangeSupport.removePropertyChangeListener(l);
		prognoseParameter = null;
	}

	/**
	 * Gibt den aktuellen Wert des Parameters der Ganglinienprognose zurück.
	 *
	 * @return der Prognoseparameter.
	 */
	public PdGanglinienModellPrognose.Daten getPrognoseParameter() {
		if (prognoseParameter != null) {
			return prognoseParameter;
		}

		return pdPrognoseParameter.abrufenDatum();

		// TODO von außen machen
		// if (getAnfrage().isZyklischePrognose()) {
		// pdPrognoseParameter.addUpdateListener(datensatzUpdateListener);
		// }
	}

}
